@@ -17,43 +17,56 @@
 #if HAVE_ERRNO_H
 #include <errno.h>
 #endif
-#if HAVE_CHFLAGS
 #include <sys/types.h>
-#include <sys/stat.h>		/* For the flag values.  */
-#else
+#include <sys/stat.h>
+#if HAVE_EXT2_IOCTLS
 #include <sys/ioctl.h>
 #endif
 
-#include <linux/ext2_fs.h>
-
 #include "e2p.h"
 
+/*
+ * Deal with lame glibc's that define this function without actually
+ * implementing it.  Can you say "attractive nuisance", boys and girls?
+ * I knew you could!
+ */
+#ifdef __linux__
+#undef HAVE_CHFLAGS
+#endif
+
 int setflags (int fd, unsigned long flags)
 {
+	struct stat buf;
 #if HAVE_CHFLAGS
-  unsigned long bsd_flags = 0;
+	unsigned long bsd_flags = 0;
 
 #ifdef UF_IMMUTABLE
-  if (flags & EXT2_IMMUTABLE_FL)
-    bsd_flags |= UF_IMMUTABLE;
+	if (flags & EXT2_IMMUTABLE_FL)
+		bsd_flags |= UF_IMMUTABLE;
 #endif
 #ifdef UF_APPEND
-  if (flags & EXT2_APPEND_FL)
-    bsd_flags |= UF_APPEND;
+	if (flags & EXT2_APPEND_FL)
+		bsd_flags |= UF_APPEND;
 #endif
 #ifdef UF_NODUMP
-  if (flags & EXT2_NODUMP_FL)
-    bsd_flags |= UF_NODUMP;
+	if (flags & EXT2_NODUMP_FL)
+		bsd_flags |= UF_NODUMP;
 #endif
 
-  return fchflags (fd, bsd_flags);
+	return fchflags (fd, bsd_flags);
 #else
 #if HAVE_EXT2_IOCTLS
-	return ioctl (fd, EXT2_IOC_SETFLAGS, &flags);
-#else /* ! HAVE_EXT2_IOCTLS */
-	extern int errno;
+	int	f;
+
+	if (!fstat(fd, &buf) &&
+	    !S_ISREG(buf.st_mode) && !S_ISDIR(buf.st_mode)) {
+		errno = EOPNOTSUPP;
+		return -1;
+	}
+	f = (int) flags;
+	return ioctl (fd, EXT2_IOC_SETFLAGS, &f);
+#endif /* HAVE_EXT2_IOCTLS */
+#endif
 	errno = EOPNOTSUPP;
 	return -1;
-#endif /* ! HAVE_EXT2_IOCTLS */
-#endif
 }
